@@ -14,7 +14,7 @@
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 
-from .common import *
+from nxt.sensor import Type, Mode
 from .digital import BaseDigitalSensor
 from .analog import BaseAnalogSensor
 
@@ -98,15 +98,15 @@ class Accelerometer(BaseDigitalSensor):
             self.x, self.y, self.z = x, y, z
     
     def __init__(self, brick, port, check_compatible=True):
-        super(Accelerometer, self).__init__(brick, port, check_compatible)
+        super().__init__(brick, port, check_compatible)
 
     def get_acceleration(self):
-        """Returns the acceleration along x, y, z axes. Units are unknown to me.
+        """Returns the acceleration along x, y, z axes. 200 => 1g.
         """
         xh, yh, zh, xl, yl, zl = self.read_value('all_data')
-        x = xh << 2 + xl
-        y = yh << 2 + yl
-        z = zh << 2 + yl
+        x = xh << 2 | xl
+        y = yh << 2 | yl
+        z = zh << 2 | zl
         return self.Acceleration(x, y, z)
     
     get_sample = get_acceleration
@@ -142,7 +142,7 @@ whether this worked for you or not!
             self.channel_4 = (m4A, m4B)
     
     def __init__(self, brick, port, check_compatible=True):
-        super(IRReceiver, self).__init__(brick, port, check_compatible)
+        super().__init__(brick, port, check_compatible)
 
     def get_speeds(self):
         """Returns the motor speeds for motors A and B on channels 1-4.
@@ -193,9 +193,10 @@ but not tested. Please report whether this worked for you or not!
         def get_dir_brightness(self, direction):
             "Gets the brightness of a given direction (1-9)."
             if direction%2 == 1: #if it's an odd number
-                exec("val = self.sensor_%d" % ((direction-1)/2+1))
+                val = getattr(self, "sensor_%d" % ((direction-1)//2+1))
             else:
-                exec("val = (self.sensor_%d+self.sensor_%d)/2" % (direction/2, (direction/2)+1))
+                val = (getattr(self, f"sensor_{direction // 2}")
+                       + getattr(self, f"sensor_{direction // 2 + 1}")) / 2
             return val
     
     class DCData(_data):
@@ -208,7 +209,7 @@ but not tested. Please report whether this worked for you or not!
             
     
     def __init__(self, brick, port, check_compatible=True):
-        super(IRSeekerv2, self).__init__(brick, port, check_compatible)
+        super().__init__(brick, port, check_compatible)
 
     def get_dc_values(self):
         """Returns the unmodulated (DC) values.
@@ -246,7 +247,7 @@ class EOPD(BaseAnalogSensor):
     _MAX_DISTANCE = 1023
 
     def __init__(self, brick, port):
-        super(EOPD, self).__init__(brick, port)
+        super().__init__(brick, port)
         from math import sqrt
         self.sqrt = sqrt
 
@@ -268,7 +269,7 @@ class EOPD(BaseAnalogSensor):
     def get_raw_value(self):
         '''Unscaled value read from sensor.'''
 
-        return self._MAX_DISTANCE - self.get_input_values().raw_ad_value
+        return self._MAX_DISTANCE - self.get_input_values().raw_value
     
     def get_processed_value(self):
         '''Derived from the square root of the raw value.'''
@@ -337,7 +338,7 @@ but not tested. Please report whether this worked for you or not!"""
             self.red, self.green, self.blue, self.white = red, green, blue, white
     
     def __init__(self, brick, port, check_compatible=True):
-        super(Colorv2, self).__init__(brick, port, check_compatible)
+        super().__init__(brick, port, check_compatible)
 
     def get_active_color(self):
         """Returns color values when in active mode.
@@ -373,7 +374,7 @@ class Gyro(BaseAnalogSensor):
 #takes the currect value and uses it to offset subsequesnt ones.
 
     def __init__(self, brick, port):
-        super(Gyro, self).__init__(brick, port)
+        super().__init__(brick, port)
         self.set_input_mode(Type.ANGLE, Mode.RAW)
         self.offset = 0
     
@@ -445,10 +446,10 @@ and can be converted into a list of bools or an integer."""
             self.a0, self.a1, self.a2, self.a3, self.a4 = a0, a1, a2, a3, a4
     
     def get_analog(self):
-        return Analog_Data(self.read_value('all_analog'))
+        return self.Analog_Data(*self.read_value('all_analog'))
     
     def get_digital(self):
-        return Digital_Data(self.read_value('digital_in')[0])
+        return self.Digital_Data(self.read_value('digital_in')[0])
     
     def set_digital(self, pins):
         """Can take a Digital_Data() object"""
@@ -471,18 +472,18 @@ the sensor but not tested. Please report whether this worked for you or not!"""
         's1pos': (0x42, 'B'),
         's2pos': (0x43, 'B'),
         's3pos': (0x44, 'B'),
-        'p4pos': (0x45, 'B'),
-        'p5pos': (0x46, 'B'),
-        'p6pos': (0x47, 'B'),
-        'pwm': (0x46, 'B'),
+        's4pos': (0x45, 'B'),
+        's5pos': (0x46, 'B'),
+        's6pos': (0x47, 'B'),
+        'pwm': (0x48, 'B'),
     })
     
     class Status:
-        RUNNING = 0x00 #all motors stopped
-        STOPPED = 0x01 #motor(s) moving
+        STOPPED = 0x00 #all motors stopped
+        RUNNING = 0x01 #motor(s) moving
 
     def __init__(self, brick, port, check_compatible=True):
-        super(ServoCon, self).__init__(brick, port, check_compatible)
+        super().__init__(brick, port, check_compatible)
 
     def get_status(self):
         """Returns the status of the motors. 0 for all stopped, 1 for
@@ -540,7 +541,7 @@ class MotorCon(BaseDigitalSensor):
             self.p, self.i, self.d = p, i, d
     
     def __init__(self, brick, port, check_compatible=True):
-        super(MotorCon, self).__init__(brick, port, check_compatible)
+        super().__init__(brick, port, check_compatible)
     
     def set_enc_target(self, mot, val):
         """Set the encoder target (-2147483648-2147483647) for a motor
@@ -604,7 +605,38 @@ MotorCon.PID_Data(p, i, d) format.
     def get_battery_voltage(self):
         """Gets the battery voltage (in millivolts/20)
         """
-        high, low = self.read_value('bateryvoltage')[0]
-        return high << 2 + low
+        data = self.read_value('batteryvoltage')
+        high = data[0]
+        low = data[1]
+        return high << 2 | low
 
 MotorCon.add_compatible_sensor(None, 'HiTechnc', 'MotorCon')
+
+
+class Angle(BaseDigitalSensor):
+    """HiTechnic Angle Sensor."""
+    I2C_ADDRESS = BaseDigitalSensor.I2C_ADDRESS.copy()
+    I2C_ADDRESS.update({
+        'mode': (0x41, 'c'),
+        'angle': (0x42, '2B'),
+        'angle_acc': (0x44, '>l'),
+        'rpm': (0x48, '>h'),
+    })
+
+    def get_angle(self):
+        v = self.read_value('angle')
+        return v[0] * 2 + v[1]
+
+    get_sample = get_angle
+
+    def get_accumulated_angle(self):
+        return self.read_value("angle_acc")[0]
+
+    def get_rpm(self):
+        return self.read_value("rpm")[0]
+
+    def calibrate(self):  #Current angle will be zero degrees written in EEPROM
+        self.write_value('mode', b'C')
+
+    def reset(self):    #Reset accumulated angle
+        self.write_value('mode', b'R')
